#include "stdafx.h"
#include "utility.h"
#include <windows.h>
#include "avisynth.h" // for AvisynthError
#include <cassert>
#include <cstddef>

// returns true if path does not contain folder names (it consists only of filename)

bool IsFilenamePath(const std::string& path)
{
	return path.find_first_of("\\/") == std::string::npos;
}

// returns true if path is an absolute path

bool IsAbsolutePath(const std::string& path)
{
	bool is_unc_path;
	if (path.size() >= 2)
		is_unc_path = (path[0] == '\\' && path[1] == '\\')
					|| (path[0] == '/' && path[1] == '/');
	else
		is_unc_path = false;
    return (path.find(':') != std::string::npos) || is_unc_path;
}

// splits fullname in path (not including final (back)slash) and filename parts
// and returns them in dirname and basename, respectively

void SplitFilename(const std::string& fullname, 
				   std::string& dirname, std::string& basename)
{
	// find the final slash //
	size_t split_pos = fullname.find_last_of("\\/"); // size_t not part of std in VC++ 6.0:
	if (split_pos != std::string::npos) // it has (back)slash
	{   
		// ***Include (back)slash*** in dirname so that can use c_str
		// at the SetCurrentDirectory system call in ChangeWorkingDir 
		// (see Windows SDK docs for details)
		dirname = fullname.substr(0, split_pos + 1); 
		basename = fullname.substr(split_pos + 1);
	}
	else
	{
		dirname.erase();
		basename = fullname;
	}
}

// Returns current working dir to cur_path. 
// May throw AvisynthError exceptions

void GetWorkingDir(std::string& cur_path) {
	unsigned long bufsize = GetCurrentDirectory(0, NULL);
	char* buffer = new char[bufsize + 2]; // add 1 for '\0' and 1 for '\\'
	if (buffer == NULL)
		throw AvisynthError("immaavs: not enough memory");
	if (GetCurrentDirectory(bufsize, buffer) == 0) {
		delete[] buffer;
		throw AvisynthError("immaavs: GetCurrentDirectory failed");
	}
	// ensure that a trailing backslash is included, 
	// so that the string can be fed to RestoreWorkingDIr
	cur_path.reserve(bufsize);
	cur_path = buffer;
	if (cur_path[cur_path.size() - 1] != '\\')
		cur_path += "\\";
	delete[] buffer;
}

// If new_path is not "", sets working dir to new_path.
// Expects new_path to end with a \ character.
// May throw AvisynthError exceptions.

void SetWorkingDir(const std::string& new_path) {
	if (new_path != "") 
	{
		// ***assumes new_path ends with a (back)slash***
		// this avoids the need for a string copy (speed optimisation)
		assert(new_path[new_path.size() - 1] == '\\');
		if (SetCurrentDirectory(new_path.c_str()) == 0)
			throw AvisynthError("immaavs: SetCurrentDirectory failed");
	}
}

// If fullname contains a folder part, changes working dir to it and returns
// old working dir to old_path, new working dir to new_path and the filename 
// part to basename.
// If fullname does *not* contain a folder part, no change is made and returns
// "" to old_path, "" to new_path and fullname to basename.
// Use SetWorkingDir(old_path) to restore the working dir.

void ChangeWorkingDir(const std::string& fullname, 
					  std::string& old_path, std::string& new_path, 
					  std::string& basename)
{
	// SetCurrentDirectory works for W2K and higher only !!!
	if (IsFilenamePath(fullname) == false)
	{
		GetWorkingDir(old_path);
		SplitFilename(fullname, new_path, basename);
		// ***assumes new_path ends with a (back)slash***
		// this avoids the need for a string copy (speed optimisation)
		assert(new_path[new_path.size() - 1] == '\\');
		if (SetCurrentDirectory(new_path.c_str()) == 0)
			throw AvisynthError("immaavs: SetCurrentDirectory failed");
	} 
	else
	{
		old_path = "";
		new_path = "";
		basename = fullname;
	}
}
